<?php

namespace GopNext\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;

class BudgetPreparationController {
  
    /**
    * Class constructor
    * 
    * Initialize an object's properties upon creation of the object.
    * 
    * @param object $environment - Allows access to session data
    * @param object $util - File containing generic functions
    * @param object $service - file that contains connections to the database
    * 
    * @return void
    * 
    * @access public
    * 
    * @author Grazielle Ferreira Antunes
    */
    public function __construct($environment, $util, $service) {
        $this->environment  = $environment;
        $this->util         = $util;
        $this->service      = $service;
    }

    /**
    * It takes some session data, checks if there are branches chosen by the user, otherwise it searches for the branches. 
    * Generates the necessary lists and executes the procedure to prepare the budget. 
    * Returns any errors written to the log table.
    * 
    * @param Request $request – Object that receives data from the caller.
    * @param Response $response –  Object that sends back data to the caller.
    * 
    * @access public
    * 
    * @author Grazielle Ferreira Antunes
    */ 
    public function prepareBudget(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        try{
            $nrOrg        = $this->environment->getNrOrgTrab();
            $currentTime  = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
            $cdOperLogado = $this->environment->getCdOperador();
            $cdFilial = [];
            
            $arrayUnits = array();
            if (count($row['CDFILIAL']) > 0) {
                $cdFilial = $row['CDFILIAL'];
                array_push($arrayUnits, $cdFilial);
            } else {
                $units = $this->selectUnit($row);
                if(!count($units)) {
                    throw new \Exception('NO_UNITS_AVAILABLE');
                }
                $cdFilial = array_column($units, 'CDFILIAL');
                array_push($arrayUnits, $units);
            }
            
            $nrListaFilial = $this->util->insertLista($cdFilial);
            $nrListaOperador = $this->util->insertLista($row['CDOPERADOR']);
            $this->service->PREPARA_ORCAMENTO($row, $nrListaOperador, $nrListaFilial, $cdOperLogado);
            
            $nrListaFilialError = $this->util->insertLista($cdFilial);
            $errorList = $this->noGenerated($row, $nrListaFilialError);
            if(count($errorList)) {
                $response->addDataSet(new DataSet("errorList", $errorList));
            }
            $this->util->deleteLista($nrListaFilialError);
            
            $response->addDataSet(new DataSet("units", $arrayUnits));
        }catch (Exception $e) {
            throw $e;
        }
    }
    
    /**
    * Prepares the parameters to execute the query that searches for units in the database and calls the function in the service.
    * 
    * @param object $row – Object that contains all parameters coming from the frontend.
    * 
    * @return array - Array with all units that meet the parameters.
    * 
    * @access public
    * 
    * @author Grazielle Ferreira Antunes
    */ 
    public function selectUnit($row) {
        try{
            $row = $row == null ? false : true;                
            $params = array (
                'CDGRUPFILI'      => $row ? (isset($row['CDGRUPFILI']) ? $row['CDGRUPFILI'] : 'T') : null,
                'CDSEGMER'        => $row ? (isset($row['CDSEGMER']) ? $row['CDSEGMER'] : 'T') : null,
                'CDOPERADOR'      => $row ? (isset($row['CDOPERADOR']) ? $row['CDOPERADOR'] : 'T') : null,
                'CDSUPERVISOR'    => $row ? (isset($row['CDSUPERVISOR']) ? $row['CDSUPERVISOR'] : 'T') : null,
                'CDREGIAO'        => $row ? (isset($row['CDREGIAO']) ? $row['CDREGIAO'] : 'T') : null
            );
            $result = $this->service->selectUnit($params);
            return $result;
        }catch (Exception $e) {
            throw $e;
        }
    }
    
    /**
    * Prepares the parameters for the query that returns the errors generated in the log. and calls the function in the service.
    * 
    * @param object $row – Object that contains all parameters coming from the frontend.
    * @param string $nrListaFilial – Number of the branch list to be used as a parameter for the query.
    * 
    * @return array - Array with the list of errors that were generated when executing the procedure.
    * 
    * @access public
    * 
    * @author Grazielle Ferreira Antunes
    */ 
    public function noGenerated($row, $nrListaFilial){
        $params =  array(
                            'NRLISTA' =>  $nrListaFilial,
                            'NRANOORCAGOP' => $row['NRANOORCAGOP'],
                            'IDPERIODOORCA'=> $row['IDPERIODOORCA'],
                            'NRPERIODOORCA'=> $row['VALUE_PERIOD']
                        );
        $result = $this->service->noGenerated($params);
		return  $result;
    }
}